/* 
  exFATUTIL DIR command implementation
  (C) 2010 Alexey Voskov
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <io.h>
#include <conio.h>


#include "exfatlib.h"

/* 
  DIR command
 */
int command_dir(exfat_fd *fd, fat_int16 *tab)
{
  exfat_fd fdbuf;
  static char fname[256];
  // Print the "head" of the table
  printf("       %10s %10s FileName\n", "Size", "FirstClust");
  // First element of the folder
  if (!exfat_findfirst(fd, &fdbuf))
  {
    printf("NO FILES!\n\n");
    return 0; 
  }
  // Next elements of the folder
  do 
  {
    unicode_to_oem(fdbuf.filename, fname, tab);
    if (fdbuf.chainflag) printf("*");
    else                 printf(" ");
    if (fdbuf.attrib & ATTR_DIR)
      printf("<DIR> ");
    else
      printf("      ");
    printf("%10" FORMAT_I64 "d 0x%.8X %s\n", fdbuf.filesize, fdbuf.firstclust, fname);
  } while (exfat_findnext(fd, &fdbuf));
  printf("(*) - fragmented files\n\n");
  return 0;
}

/* 
   Reads the file to the disk
 */
int exfat_getfile(exfat_fd *fd, char *outfname)
{ int fdout;
  fat_int32  tlen;
  fat_int64 n, ind;
  // Create the output file
  fdout = open(outfname, O_RDWR | O_BINARY | O_CREAT | O_TRUNC, 0666);
  if (fdout == -1)
  { 
    return 1;
  }
  // Check input file size
  if (fd->filesize == 0)
  { close(fdout); // Empty file - work finished
    return 0;
  }
  // Calculate the number of sectors
  n = (fd->filesize) >> (fd->pd->vbr.bytepersect);
  tlen = (__uint32)(fd->filesize % fd->pd->bps);
  if (fd->filesize % fd->pd->bps)
    n++;
  // Copy all sectors to the file
  lseek(fdout, 0, SEEK_SET); 
  for (ind = 0; ind < n; ind++)
  { 
    exfat_readfilesector(fd, ind, NULL);
    if (ind == (n - 1) && tlen != 0)
    { // Tail shorter than one sector
      write(fdout, fd->pd->filebuf, tlen);
    }
    else
    { // Read the whole sector
      write(fdout, fd->pd->filebuf, fd->pd->bps);
    }
  }
  // Close the output file
  close(fdout);
  return 0;
}




int main(int argc, char *argv[])
{
  int drvnum = 0, partnum = 0, showpart = 0, i;
  fat_int32 lbaadr[32], psize[10];
  fat_int16 tab[128], ubuf[256];
  char buf[256];
  exfat_descriptor pd;
  exfat_fd fd_root, fd;

  // Show information about the command
  if (argc == 1)
  {
    printf(EXFATINFO);
    printf("EFDIR - DIR command\n  Usage:\n");
    printf("  EFDIR 00:\\DOS\\EXFAT\n");
    printf("  00 - partition number (first digit - physical drive number\n");
    printf("                         second digit - partition number)\n");
    printf("  EFDIR 0 - show partitions of physical drive 0\n\n");
    return 0;    
  }

  // Get drive number from argument 1
  if (strlen(argv[1]) >= 1 && isdigit(argv[1][0]))
  {
    drvnum = argv[1][0] - '0';
    if (strlen(argv[1]) == 1) 
      showpart = 1;
    else 
      showpart = 0;
  }
  else
  {
    printf("Invalid format of directory name\n");
    return 1;
  }

  // Mount exFAT partitions
  find_exfat_partitions(drvnum, lbaadr, psize);
  if (lbaadr[0] == 0)
  { // No exFAT partitions on the physical drive drvnum
    printf("exFAT partitions not found!\n");
    return 0;
  }

  else // Select the partition
  { // Calculate the number of partitions 
    // (and show their list if required
    if (showpart)
    { printf("exFAT partitions\n");
      for (i = 0; lbaadr[i] !=0; i++) 
        printf("%d: %.8X (%d Mb)\n", i, lbaadr[i], psize[i]/2048);
      return 0;
    }

     // Select the partition
    if (!isdigit(argv[1][1]))
      {
        printf("Invalid format of directory name\n");
        return 1;
      }

     partnum = argv[1][1] - '0';
     if (lbaadr[partnum] == 0)
        {
          printf("Cannot find exFAT partition %d%d:\n", drvnum, partnum);
          return 0;
        }
    
    // Mount partition
	printf("mounting ExFAT at %x\n", lbaadr[partnum]);

    exfat_mount(drvnum, lbaadr[partnum], &pd);			//!!
  }
  // Load UNICODE table
  if (unicode_loadtab("cp866uni.tbl", tab))
  {
    printf("Cannot load OEM to UNICODE table\n");
    printf("Default table will be used\n\n");
    for (i = 0; i < 128; i++)
      tab[i] = '_';
  }
  // Root directory
  fd_root.firstclust = pd.vbr.rootclust;
  fd_root.chainflag  = 1;
  fd_root.attrib = ATTR_DIR;
  fill_exfat_fd(&pd, &fd_root);
  // Execute DIR command
  oem_to_unicode(argv[1], ubuf, tab);
  if (exfat_parsepath(ubuf, &fd_root, &fd) == 0)
  {
    if (fd.attrib & ATTR_DIR)
	{	  
      command_dir(&fd, tab);
    }
    else
    {
      printf("File %s is not a directory\n", argv[1]);
      printf("Do you want to copy it to the current folder(Y/N)?\n");
      i = getch();
      if (i == 'Y' || i == 'y') 
      { printf("Copying...\n");
        unicode_to_oem(fd.filename, buf, tab);
        exfat_getfile(&fd, buf);
      }
    }
  }
  else
  {
    printf("Directory %s not found", argv[1]);
  }
  // Unmount partition
  exfat_unmount(&pd);
  // Successful finish
  return 0;
}
