#ifndef __FS_EXFATLIB_H
#define __FS_EXFATLIB_H
#include "fatdtype.h"
#include "diskio.h"
#include "fs_syms.h"

// exFATUTIL banner
#define EXFATINFO "exFATUTIL v0.01\n(C) 2010 Alexey Voskov\n\n"

// Codes of partitions required for exFAT search
#define PART_EXFAT 0x07
#define PART_LOGIC 0x0F
#define PART_LOGICOLD 0x05

// Special numbers of clusters (in FAT entries)
#define EXFAT_EOF 0xFFFFFFFF // EOF exFAT entry
#define EXFAT_BAD 0xFFFFFFF7 // Bad block exFAT entry

// Directory entries codes
#define DE_END    0x00 // End of directory (folder) 
#define DE_FILE   0x85 // Beginning of file description (attributes, data and time)
#define DE_STREAM 0xC0 // Data about file name, size and location
#define DE_FNAME  0xC1 // Entry with the (part) of file name

// File attributes masks
#define ATTR_ARC  0x20 // Archive
#define ATTR_DIR  0x10 // Directory
#define ATTR_SYS  0x04 // System
#define ATTR_HID  0x02 // Hidden
#define ATTR_RO   0x01 // Read-only

// Functions
int ifexfat(int drvnum, fat_int32 lbaadr);
int find_exfat_partitions(int drvnum, fat_int32 *lbaadr_v, fat_int32 *psize_v);
int exfat_mount(int drvnum, fat_int32 lbaadr, exfat_descriptor *fd);
int exfat_unmount(exfat_descriptor *exfat);
fat_int32 getexfatentry(exfat_descriptor *fd, fat_int32 ind);
int exfat_readfilesector(exfat_fd *fd, fat_int64 ind, char *buf);
char *exfat_getfolderentry(exfat_fd *fd);
int exfat_findnext(exfat_fd *fd, exfat_fd *outfd);
int exfat_findfirst(exfat_fd *fd, exfat_fd *outfd);
int exfat_findfile(exfat_fd *fd, exfat_fd *outfd, fat_int16 *filename);
int exfat_parsepath(fat_int16 *path, exfat_fd *fd_root, exfat_fd *fd);
#endif
