#include "fatdtype.h"
#include <memory.h>


fat_int64 getint64(char *buf)
{
#ifdef _MSC_VER
	return *(fat_int64*)buf;
#else
  fat_int64 res;
  res  = (fat_int64) buf[0];
  res |= (fat_int64) buf[1] << 8;
  res |= (fat_int64) buf[2] << 16;
  res |= (fat_int64) buf[3] << 24;
  res |= (fat_int64) buf[4] << 32;
  res |= (fat_int64) buf[5] << 40;
  res |= (fat_int64) buf[6] << 48;
  res |= (fat_int64) buf[7] << 56;
  return res;
#endif
}

fat_int32 getint32(char *buf)
{ 
#ifdef _MSC_VER
	return *(fat_int32*)buf;
#else
  fat_int32 res;
  res  = (fat_int32) buf[0];
  res |= (fat_int32) buf[1] << 8;
  res |= (fat_int32) buf[2] << 16;
  res |= (fat_int32) buf[3] << 24;
  return res;
#endif
}

fat_int16 getint16(char *buf)
{
#ifdef _MSC_VER
	return *(fat_int16*)buf;
#else
  fat_int16 res;
  res  = (fat_int16) buf[0];
  res |= (fat_int16) buf[1] << 8;
  return res;
#endif
}

/*
   Parses the VBR of EXFAT and records the result
   in the structure
 */
void parsevbr(exfat_vbr *vbr, char *buf)
{
  memcpy(vbr->jmpboot,buf,3);
  memcpy(vbr->fsname,buf+3,8);
  memcpy(vbr->reserved1,buf+11,53);

  vbr->partoffset   = getint64(buf+64);
  vbr->vollength    = getint64(buf+72);
  vbr->fatoffset    = getint32(buf+80);
  vbr->fatlength    = getint32(buf+84);
  vbr->heapoffset   = getint32(buf+88);
  vbr->clustnum     = getint32(buf+92);
  vbr->rootclust    = getint32(buf+96);
  vbr->volserial    = getint32(buf+100);
  vbr->fsrev        = getint16(buf+104);
  vbr->volflags     = getint16(buf+106);
  vbr->bytepersect  = buf[108];
  vbr->sectperclust = buf[109];
  vbr->numoffats    = buf[110];
  vbr->driveselect  = buf[111];
  vbr->percentused  = buf[112];

  memcpy(vbr->reserved2,buf+113,7);
  
}

/* 
   Autocompletes exfat_fd structure 
   (low-level reading of the file sectors)

   Required fields in exfat_fd structure:   
     filesize, firstclust, clustnum, chainflag
   Will be autocompleted
     pd, pos, cc_ind, cc_num
 */
void fill_exfat_fd(exfat_descriptor *pd, exfat_fd *fd)
{
  fd->pd     = pd;
  fd->pos    = 0;
  fd->cc_ind = fd->firstclust;
  fd->cc_num = 0;
}

