#ifndef __FATDTYPE_H
#define __FATDTYPE_H
/*----- Data types -----*/

#ifdef _MSC_VER 
	#define FORMAT_I64 "I64"			    // for printf("%I64u",...
	#define __uint32  unsigned __int32
	#define __uint64  unsigned __int64
#else
	#define FORMAT_I64 "ll"					// for printf("%llu",...
	#define __uint32  unsigned long
	#define __uint64  unsigned long long
#endif


#define fat_int64 __uint64    //long long int
#define fat_int32 __uint32
#define fat_int16 unsigned short
#define fat_int8  unsigned char

/*----- Structures -----*/
/* EXFAT VBR data (in the beginning of the sector) */
typedef struct {
    fat_int8  jmpboot[3];    // 00 Jump boot
    fat_int8  fsname[8];     // 03 File system name
    fat_int8  reserved1[53]; // 0b Reserved (must be zero)
    fat_int64 partoffset;    // 40 Partition offset (in sectors)
    fat_int64 vollength;     // 48 Volume length (in sectors)
    fat_int32 fatoffset;     // 50 FAT offset (in sectors)
    fat_int32 fatlength;     // 54 FAT length (in sectors)
    fat_int32 heapoffset;    // 58 Cluster heap offset (in sectors)
    fat_int32 clustnum;      // 5c Cluster count
    fat_int32 rootclust;     // 60 First cluster of root directory
    fat_int32 volserial;     // 64 Volume serial number
    fat_int16 fsrev;         // 68 File system revision 
    fat_int16 volflags;      // 6a Volume flags
    fat_int8  bytepersect;   // 6c Bytes per sector (2^bps)
    fat_int8  sectperclust;  // 6d Sectors per cluster
    fat_int8  numoffats;     // 6e Number of fats
    fat_int8  driveselect;   // 6f Used by int 13h
    fat_int8  percentused;   // 70 Percent of heap used
    fat_int8  reserved2[7];  // 71 Reserved
							 // 78..1fd payload
							 // 1fe..1ff constant '55 AA'
} exfat_vbr;

struct {
	char x [ sizeof(exfat_vbr) == 0x78 ? 1 : -1];
	} verify_size_of_exfat_vbr;


/* exFAT descriptor for mounting exFAT file system */
typedef struct
{ // Fields not from VBR
  int        drvnum;     // Drive (physical disk) number
  int        bps;        // Bytes per sector
  int        spc;        // Sectors per cluster
  fat_int32  lbaadr;     // LBA address from MBR
  // Pointers to the buffers
  char       *fatbuf;    // Sector buffer for FAT entries
  fat_int32  fat_cs;     // Current sector of FAT (it is in the buffer)
  char       *filebuf;   // Buffer for the files
  fat_int32  file_cs;    // Current sector of the file (it is in the buffer)
  // Field from VBR
  exfat_vbr  vbr;
} exfat_descriptor;

/* Internal structure for opening the file */
typedef struct {
    // Pointer to the exFAT partition descriptor
    exfat_descriptor *pd; 
    // File parameters
    fat_int64     filesize;   // File size, bytes
    fat_int32     firstclust; // First cluster
    fat_int32     clustnum;   // Number of clusters
    int           chainflag;  // Chain of clusters present/absent
    fat_int64     pos;        // Position in file
    // Extra file parameters (from exFAT entry)
    fat_int16     *filename;  // Pointer to the file name (ASCIIZ unicode)
    fat_int16     attrib;     // File attributes
    // Extra data for low-level access
    fat_int32     cc_ind;     // Index of current cluster in FAT
    fat_int32     cc_num;     // Number of current cluster 
                              // (0 - first cluster, 1 - second...)
} exfat_fd;

/*----- Functions ------*/
fat_int64 getint64(char *buf);
fat_int32 getint32(char *buf);
fat_int16 getint16(char *buf);
void parsevbr(exfat_vbr *vbr, char *buf);
void fill_exfat_fd(exfat_descriptor *pd, exfat_fd *fd);
#endif
