/* 
   EFATUTIL - exFAT utilites
   Functions for working with unicode

   (C) 2010 Alexey Voskov   
 */

#include "fs_syms.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <io.h>

/* 
  Convert UNICODE string to OEM string using codepage
  from the array 

  Input string must be finished with \0 symbol

  *inp - input UNICODE string
  *out - output UNICODE string
  *tab - codepage table generated by unicode_loadtab function
         (if it is NULL, the non-ASCII symbol 
          will be replaced by '_')
 */
void unicode_to_oem(fat_int16 *inp, char *out, fat_int16 *tab)
{ int i;
  while (*inp != 0)
  {
    if ((unsigned) *inp < 128)
      *out = (unsigned char)*inp; // Don't recode
    else
    { // Recode UNICODE to OEM
      *out = '_';
      if (tab != NULL)
      { 
        for (i = 0; i < 127; i++)   
          if (*inp == tab[i])
          { *out = i + 128;
            break;
          }
      }
    }
    // To the next symbol
    inp++; out++;
  }
  // Write 0x00 to the end of the string
  *out = 0;
}

/*
  Convert OEM string to UNICODE string using codepage
  from the array

  Input string must be finished with \0 string

 */
void oem_to_unicode(char *inp, fat_int16 *out, fat_int16 *tab)
{
  while (*inp != 0)
  {
    if ((unsigned) *inp < 128)
      *out = *inp; // Don't recode
    else
    { // Recode OEM to UNICODE
      *out = '_';
      if (tab != NULL)
      {
        *out = tab[*inp-128];
      }
    }
    // To the next symbol
    inp++; out++;
  }
  // Write 0x00 to the end of the string
  *out = 0;
}


/* 
  Load OEM to UNICODE conversion table
 */
int unicode_loadtab(char *filename, fat_int16 *tab)
{
  int fd, i, j, len;
  char *buf;
  // Open the file with UNICODE table
  fd = open(filename, O_RDWR | O_BINARY, 0666);
  if (fd == -1)
  { 
    return 1;
  }
  // Calculate the length of file
  j = lseek(fd, 0, SEEK_END);
  i = lseek(fd, 0, SEEK_SET);
  len = j - i;
  // Read the file to memory
  buf = malloc(len);
  read(fd, buf, len);
  // Analyse the file
  i = 0; 
  for (i = 0; i < len && buf[i] != 0x0A; i++) {}
  if (i == len - 1)
    return 2; // Invalid format of the file
  else
  { i+=2;
    // !!! FIXME CHECK THE LENGTH OF THE FILE AND ID
    for (j = 0; j < 128; j++)
      tab[j] = getint16(buf+i+j*2);
  }
  // Free memory
  close(fd);
  free(buf);
  return 0;
}

/*
  Compare two unicode strings
 */
int unicode_strcmp(fat_int16 *str1, fat_int16 *str2)
{
  while (*str1 != 0)
  {
    if      (*str1 > *str2) return 1;
    else if (*str1 < *str2) return -1;
    str1++; str2++;
  }
  // Check the end of the strings
  if      (*str1 > *str2) return 1;
  else if (*str1 < *str2) return -1;
  return 0;
}

